use clap::Parser;

use serialport::{DataBits, FlowControl, Parity, SerialPort, StopBits};

use std::io::{self, ErrorKind};
use std::thread;
use std::time::Duration;

const BAUD_RATE: u32 = 250000;

const MAX_SLOTS: usize = 513;

const MICROS_PER_SLOT: u64 = 44;

#[derive(Debug, Parser)]
struct Args {
    #[arg(short, long, conflicts_with = "port")]
    list_ports: bool,

    #[arg(required_unless_present = "list_ports")]
    port: Option<String>,
}

type Packet = [u8; MAX_SLOTS];

fn main() {
    let args = Args::parse();

    if args.list_ports {
        for port in serialport::available_ports().unwrap() {
            println!("{}", port.port_name);
        }

        return;
    }

    let mut serial = serialport::new(args.port.unwrap(), BAUD_RATE)
        .data_bits(DataBits::Eight)
        .flow_control(FlowControl::None)
        .parity(Parity::None)
        .stop_bits(StopBits::Two)
        .timeout(Duration::from_micros(MICROS_PER_SLOT))
        .open()
        .unwrap();

    // ignore first (possibly incomplete) packet
    let _ = read_packet(&mut serial).unwrap();

    println!("{:#?}", read_packet(&mut serial).unwrap());
}

fn read_packet(serial: &mut Box<dyn SerialPort>) -> Result<Packet, io::Error> {
    let mut buf = [0; MAX_SLOTS];
    let mut pos = 0;

    loop {
        match serial.read(&mut buf[pos..]) {
            Ok(bytes_read) => {
                pos += bytes_read;

                if pos == MAX_SLOTS {
                    break;
                }

                continue;
            }
            Err(err) if err.kind() == ErrorKind::Interrupted => continue,
            Err(err) if err.kind() == ErrorKind::TimedOut => {
                if pos > 0 {
                    break;
                }

                thread::sleep(Duration::from_micros(MICROS_PER_SLOT));
                continue;
            }
            Err(err) => return Err(err),
        }
    }

    Ok(buf)
}
